@@ -546,7 +546,22 @@ class DatabaseManagement(Base):
         meta_dask = meta_dask.astype(meta)
         meta_dask = meta_dask.set_index(pk_column)
         return meta_dask
+    
+    def _get_order_meta_data(self, meta, pk_column, query_function, con_str):
+        possible_divisions = self._get_possible_str_division()
+        if meta[pk_column] in ['str', 'string']:
+            ddf = dd.read_sql_query(sql=query_function(),  con=con_str, index_col=pk_column, divisions=possible_divisions)
+        else:
+            ddf = dd.read_sql_query(sql=query_function(),  con=con_str, index_col=pk_column, npartitions=1)
+        columns_order = ddf.columns                    
         
+        # to reorder metadata for dask          
+        try:
+            reordered_meta = [ {'column': column, 'type': meta[column] } for column in columns_order]
+            reordered_meta.append({'column': pk_column, 'type': meta[pk_column]}) # add pk meta at the end
+        except KeyError as ke:
+            raise KeyError(f"{ke}. This column is exist in data source but not exist in reflection schema. Maybe this is a new column in data source. Please check your reflection schema file and regenreate it")
+        return reordered_meta
 
     
     def write_sql_query(self, query_function=None, directory_id=None, database_id=None, pk_column=None, 
@@ -597,9 +612,42 @@ class DatabaseManagement(Base):
             desciption = f"query of {name}"
     
         _replace = self._check_fileExists(directory_id, name) if replace is None else replace
-                    
-        # to reorder metadata for dask
-        reordered_meta = [ {'column': column, 'type': type } for column, type in meta.items()]
+        
+        database_meta, schema = self.get_database_schema(database_id=database_id)
+        con_str = database_meta['sqlalchemy_uri']
+        
+        if inspect.isfunction(query_function):            
+            query_function_str = inspect.getsource(query_function)
+            query_function_function = query_function
+        elif type(query_function) == str:
+            query_function_str = query_function
+            
+            exec(f"""
+{schema}
+from sqlalchemy import sql
+{query_function}
+            """, globals())
+            query_function_function = query        
+        # import pdb; pdb.set_trace()
+        # print(pk_column)
+        
+        reordered_meta = self._get_order_meta_data(
+            meta=meta, 
+            pk_column=pk_column, 
+            query_function=query, 
+            con_str=con_str,
+        )
+#         possible_divisions = self._get_possible_str_division()
+#         if meta[pk_column] in ['str', 'string']:
+#             ddf = dd.read_sql_query(sql=query(),  con=con_str, index_col=pk_column, npartitions=1, divisions=possible_divisions)
+#         else:
+#             ddf = dd.read_sql_query(sql=query(),  con=con_str, index_col=pk_column, npartitions=1)
+#         columns_order = ddf.columns                    
+        
+#         # to reorder metadata for dask
+#         # reordered_meta = [ {'column': column, 'type': type } for column, type in meta.items()]        
+#         reordered_meta = [ {'column': column, 'type': meta[column] } for column in columns_order]
+#         reordered_meta.append({'column': pk_column, 'type': meta[pk_column]}) # add pk meta at the end
 
         res = requests.post(f"{self._base_discovery_api}/api/sql/query/", headers=self._jwt_header,
                           json={
@@ -608,13 +656,18 @@ class DatabaseManagement(Base):
                                 "name": name,
                                 "description": desciption,
                                 "pk_column": pk_column,
-                                "query": inspect.getsource(query_function) if inspect.isfunction(query_function) else query_function,
+                                "query": query_function_str,
                                 "meta": reordered_meta,
                                 "replace": _replace
                           }
         )
         return res.json() if res.status_code < 500 else res.content
-        
+    
+    def _get_possible_str_division(self):
+        possible_divisions = list('0123456789' + string.ascii_lowercase + string.ascii_uppercase)
+        possible_divisions.sort()
+        return possible_divisions
+    
     def read_sql_query(self, database_id=None, query_function=None, pk_column=None, meta=None):
         """_summary_
 
@@ -646,12 +699,10 @@ class DatabaseManagement(Base):
             raise Exception(f"Please input data `meta`=<dict>, but got {type(meta)} eg meta='{{'id':'Int64'}}' ")
         
         database_meta, schema = self.get_database_schema(database_id=database_id)
-        # con_str = self._get_connection_str(con_id=table_meta['database'])
         con_str = database_meta['sqlalchemy_uri']
         df_meta = self._create_pandas_meta(meta=meta, pk_column=pk_column)
-        if meta[pk_column] == 'str':
-            possible_divisions = list('0123456789' + string.ascii_lowercase + string.ascii_uppercase)
-            possible_divisions.sort()
+        if meta[pk_column] in ['str', 'string']:
+            possible_divisions = self._get_possible_str_division()
             ddf = dd.read_sql_query(
                 sql=query_function(), 
                 con=con_str, 
@@ -659,7 +710,7 @@ class DatabaseManagement(Base):
                 meta=df_meta, 
                 divisions=possible_divisions
             )
-        else:
+        else:            
             ddf = dd.read_sql_query(
                 sql=query_function(), 
                 con=con_str, 
